@@ -309,22 +309,16 @@ thunar_application_dbus_name_lost_cb (GDBusConnection *connection,
                                       const gchar     *name,
                                       gpointer         user_data)
 {
-  ThunarApplication *application = user_data;
-  g_critical (_("Name '%s' lost on the message dbus, exiting."), name);
-  g_application_quit (G_APPLICATION (application));
+  g_warning (_("Name '%s' lost on the message dbus."), name);
 }
 
 
 
 /* TODO: [GTK3 Port] Check if there's a cleaner way to register */
-/* this extra dbus name (besides org.xfce.Thunar) */
+/* these extra dbus names (besides org.xfce.Thunar) */
 static void
 thunar_application_dbus_init (ThunarApplication *application)
 {
-    /* Do not atempt to register if running as root */
-    if (geteuid() == 0)
-      return;
-
     application->dbus_owner_id_xfce = g_bus_own_name (G_BUS_TYPE_SESSION,
                                       "org.xfce.FileManager",
                                       G_BUS_NAME_OWNER_FLAGS_NONE,
